from common.entity.relation_key import RelationKey
from common.util.date_util import current_time


class TestRelationKey:
    def test_relation_key(self):
        relation_key = RelationKey('test', 'sd', current_time())
        print(relation_key)


from dataclasses import dataclass
from common.entity.timeline import Timeline

DEFAULT_DOCUMENT_AVG_LEN = 256


@dataclass
class Document:
    dataset_id: str
    uuid: str
    medical_institution_code: str
    medical_record_no: str
    discharge_time: str
    admission_time: str
    table_name: str
    column_name: str
    table_uuid: str
    page: int
    page_uuid: str
    start_index: int
    end_index: int
    content: str
    md5_sum: str
    timeline: Timeline = None
    global_id: str = ''

    def package_relation_key(self):
        return self.medical_institution_code + "|" + self.medical_record_no + "|" + self.discharge_time.replace("-", "")\
            .replace(" ", "").replace(":", "")



from common.service.iccc3 import convert_iccc3_name_to_iccc3_code, convert_iccc3_name_to_iccc3_code_best, \
    get_large_group, extract_iccc3_name_large_group


class TestIccc3Util:
    def test_convert_iccc3_name_to_iccc3_code(self):
        result = convert_iccc3_name_to_iccc3_code("结肠癌")
        assert result == 'Ⅺ.f2'
        result = convert_iccc3_name_to_iccc3_code("恶形黑色素瘤")
        assert result != 'Ⅺ.d'

    def test_convert_iccc3_name_to_iccc3_code_best(self):
        result = convert_iccc3_name_to_iccc3_code_best("恶形黑色素瘤")
        assert result == 'Ⅺ.d'

        result = convert_iccc3_name_to_iccc3_code_best("前体细胞白雪病")
        assert result == 'Ⅰ.a1'

    def test_get_large_group(self):
        result = get_large_group("Ⅰ.a1")
        assert result == 'Ⅰ'

    def test_extract_iccc3_name_large_group(self):
        result = extract_iccc3_name_large_group("前体细胞白雪病")
        assert result == 'Ⅰ'




import collections
import uuid

from common.service.medical_text import extract_all_relation_key, extract_medical_text, sentence_count, \
    random_pick_relation_key
from common.util.csv_util import write_csv
from common.util.database_connection_util import get_connection_by_schema
from common.util.date_util import current_time
from common.util.sql_util import store_by_batch
from common.util.string_util import cut_by_time


class TestSqlUtil:
    def test_extract_all_relation_key(self):
        pass
        # conn = get_tumour_stage_connection(b'')
        # cursor = conn.cursor()
        # relation_key_list = extract_all_relation_key(cursor)
        # for relation_key in relation_key_list:
        #     print(relation_key)

    def test_extract_medical_text(self):
        present_dict = {
            "nc_admission_record": ["nc_present_illness_history"]
        }
        conn = get_connection_by_schema(b'', 'ncpcs_all', 'prod_test')
        nlp_conn = get_connection_by_schema(b'', 'ncpcs_nlp')
        corpus_conn = get_connection_by_schema(b'', 'nlp_corpus')

        cursor = conn.cursor()
        entity_data_list = []
        entity_list = []
        admission_record_list = []
        # relation_key_list = extract_all_relation_key(cursor)[1000:1500]
        relation_key_list = extract_all_relation_key(cursor)
        for relation_key in relation_key_list:
            medical_text_list = extract_medical_text(cursor, relation_key, column_dict=present_dict, split_method=cut_by_time)
            if not medical_text_list:
                continue
            medical_text = medical_text_list[0]
            if not medical_text['文本列表']:
                continue

            entity_data_list.append({
                'nc_key': str(relation_key),
                'nc_medical_institution_code': relation_key.medical_institution_code,
                'nc_medical_record_no': relation_key.medical_record_no,
                'nc_discharge_time': relation_key.discharge_time,
                'nc_valid_flag': 1
            })

            admission_record_list.append({
                'nc_rid': str(uuid.uuid4()),
                'nc_medical_institution_code': relation_key.medical_institution_code,
                'nc_medical_record_no': relation_key.medical_record_no,
                'nc_discharge_time': relation_key.discharge_time,
                'nc_present_illness_history': medical_text['原文'],
                'nc_backup_time': current_time(),
            })
            for timeline, _ in medical_text['文本列表']:
                if not timeline:
                    continue
                entity_list.append({
                    'nc_key': str(relation_key),
                    'nc_medical_institution_code': relation_key.medical_institution_code,
                    'nc_medical_record_no': relation_key.medical_record_no,
                    'nc_discharge_time': relation_key.discharge_time,
                    'nc_entity_name': timeline.base_time,
                    'nc_entity_name_convert': timeline.base_time_convert,
                    'nc_content': timeline.content,
                    # 'nc_entity_classification': timeline.base_time_convert,
                    'nc_entity_classification': '时间',
                    'nc_table_name': medical_text['表名'],
                    'nc_column_name': medical_text['字段名'],
                    'nc_source': medical_text['表名'] + '-' + medical_text['字段名'],
                    'nc_start_index': timeline.start_index,
                    'nc_end_index': timeline.end_index,
                    'nc_page': medical_text['页码'],
                    'nc_page_uuid': medical_text['组号']
                })
        store_by_batch('nc_entity_data', entity_data_list, nlp_conn)
        store_by_batch('nc_entity', entity_list, nlp_conn)
        store_by_batch('nc_admission_record', admission_record_list, corpus_conn)

    def test_sentence_count(self):
        sentence_list = ["测试", "测试", "四大皆空", "第三方i哦", "第三方i哦", "第三方i哦"]
        sentence_count(sentence_list)

    def test_random_pick_relation_key(self):
        conn = get_connection_by_schema(b'')
        cursor = conn.cursor()
        all_relation_key_list = extract_all_relation_key(cursor)
        relation_key_dict = collections.defaultdict(list)
        for relation_key in all_relation_key_list:
            relation_key_dict[relation_key.medical_institution_code].append(relation_key)
        assert len(random_pick_relation_key(relation_key_dict, 500)) == 500
        assert len(random_pick_relation_key(relation_key_dict, 37)) == 37




import paramiko


def execute_remote_command(host, port, username, password, command):
    try:
        # 创建SSH客户端对象
        client = paramiko.SSHClient()

        # 自动添加主机密钥
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        # 连接到远程主机
        client.connect(hostname=host, port=port, username=username, password=password)

        # 执行命令
        stdin, stdout, stderr = client.exec_command(command)

        # 打印命令输出
        print("命令输出:")
        for line in stdout:
            print(line.strip())

        # 打印错误输出
        error = stderr.read().decode()
        if error:
            print("错误信息:")
            print(error)

        # 关闭SSH连接
        client.close()

    except Exception as e:
        print("连接远程主机并执行命令时出错:", e)




import csv


# 输出数据到data.csv文件
def write_csv(csvheader, results, fileName):
    with open(fileName, 'w', encoding='utf-8-sig') as f:
        # f.write(codecs.BOM_UTF8)
        writer = csv.DictWriter(f, csvheader)
        writer.writeheader()
        for row in results:
            writer.writerow(row)


import hashlib
import random
import re
from collections import Counter
from common.constants.level_dict import COLUMN_DICT, NAME_MAP
from common.entity.document import Document, DEFAULT_DOCUMENT_AVG_LEN
from common.entity.relation_key import RelationKey
from common.util.date_util import current_time
from common.util.sql_util import store_by_batch
from common.util.string_util import cut_by_time

ORDER_TABLE_DICT = {
    "nc_admission_record": {
        "pageUuid": "",
        "orderSql": "",
        "tableUuid": "703000000"
    },
    "nc_discharge_record": {
        "pageUuid": "",
        "orderSql": "",
        "tableUuid": "707000000"
    },
    "nc_daily_disease_course": {
        "pageUuid": "706010000",
        "orderSql": " order by nc_disease_course_no ASC,sort_time ASC, nc_record_time, nc_rid",
        "tableUuid": "706000000"
    },
    "nc_pathology_info": {
        "pageUuid": "711010000",
        "orderSql": " order by nc_pathology_no ASC,sort_time ASC",
        "tableUuid": "711000000"
    },
    "nc_imageology_exam": {
        "pageUuid": "713010000",
        "orderSql": " order by nc_exam_order ASC,nc_report_no ASC,sort_time ASC",
        "tableUuid": "713000000"
    },
    "nc_fist_disease_course": {
        "pageUuid": "705010000",
        "orderSql": " order by nc_disease_course_no ASC,sort_time ASC",
        "tableUuid": "705000000"
    },
    "nc_24hours_admission_discharge_info": {
        "pageUuid": "",
        "orderSql": "",
        "tableUuid": "708000000"
    },
    "nc_readmission_record": {
        "pageUuid": "",
        "orderSql": "",
        "tableUuid": "709000000"
    },
    "nc_death_record": {
        "pageUuid": "",
        "orderSql": "",
        "tableUuid": "712000000"
    }
}

GET_TEXT_SQL_FORMAT = "select {} from {} where nc_medical_institution_code = '{}' and nc_medical_record_no = '{}' and " \
                      "nc_discharge_time = '{}' and nc_hedge = 0 and nc_data_status != 99"
DATASET_ID_SCREEN_SQL_FORMAT = " and dataset_id = '{}'"


def generate_dataset_screen_sql(dataset_id, sql):
    if dataset_id:
        return sql + DATASET_ID_SCREEN_SQL_FORMAT.format(dataset_id)
    return sql


def extract_document_by(cursor, medical_institution_code, medical_record_no, discharge_time, admission_time=None,
                        column_dict=COLUMN_DICT, avg_len=DEFAULT_DOCUMENT_AVG_LEN, dataset_id=None):
    if not admission_time:
        admission_time = discharge_time
    relation_key = RelationKey(medical_institution_code, medical_record_no, discharge_time, admission_time)
    return extract_medical_text(cursor, relation_key, column_dict, avg_len, dataset_id)


def extract_medical_text(cursor, relation_key, column_dict=COLUMN_DICT, avg_len=DEFAULT_DOCUMENT_AVG_LEN,
                         dataset_id=None):
    document_list = []
    for table_name, column_list in column_dict.items():
        get_text_sql = GET_TEXT_SQL_FORMAT.format(','.join(column_list), table_name,
                                                  relation_key.medical_institution_code,
                                                  relation_key.medical_record_no, relation_key.discharge_time)
        get_text_sql = generate_dataset_screen_sql(dataset_id, get_text_sql)
        table_uuid, page_uuid, get_text_sql = generate_order_sql(get_text_sql)
        cursor.execute(get_text_sql)
        page = 1
        for ele in cursor.fetchall():
            for i, val in enumerate(ele):
                if not val:
                    continue
                column_name = column_list[i]
                content_list = cut_by_time(val, admission_time=relation_key.admission_time, avg_len=avg_len)
                for start_index, end_index, timeline in content_list:
                    content = val[start_index:end_index]
                    document = Document(dataset_id, '', relation_key.medical_institution_code,
                                        relation_key.medical_record_no,
                                        relation_key.discharge_time, relation_key.admission_time, table_name,
                                        column_name, table_uuid, page, page_uuid, start_index,
                                        start_index + len(content),
                                        content, '', timeline, '')
                    document_list.append(document)

            page += 1
    calculate_document_list(document_list)
    return document_list


def write_document_list_to_db(conn, document_list):
    cur_time = current_time()
    store_list = []
    for document in document_list:
        store_list.append({
            'dataset_id': document.dataset_id,
            'nc_uuid': document.uuid,
            'nc_add_account': '机器自动识别',
            'nc_upd_account': '机器自动识别',
            'nc_add_time': cur_time,
            'nc_upd_time': cur_time,
            'nc_global_id': document.global_id,
            'nc_key': document.package_relation_key(),
            'nc_medical_institution_code': document.medical_institution_code,
            'nc_medical_record_no': document.medical_record_no,
            'nc_discharge_time': document.discharge_time,
            'nc_admission_time': document.admission_time,
            'nc_table_name': document.table_name,
            'nc_column_name': document.column_name,
            'nc_source': '{}-{}'.format(NAME_MAP.get(document.table_name), NAME_MAP.get(document.column_name)),
            'nc_content': document.content,
            'nc_md5_sum': document.md5_sum,
            'nc_start_index': document.start_index,
            'nc_end_index': document.end_index,
            'nc_page': document.page,
            'nc_page_uuid': document.page_uuid,
            'nc_table_uuid': document.table_uuid,
        })
    store_by_batch("nc_document", store_list, conn)


def get_document_by(cursor, document_uuid, dataset_id=None):
    document_list = get_document_list_by(cursor, [document_uuid], dataset_id)
    if document_list:
        return document_list[0]
    return None


def get_document_list_by(cursor, document_uuid_list, dataset_id=None):
    query_sql = ','.join(["'{}'".format(uuid) for uuid in document_uuid_list])
    get_document_list_sql = "select dataset_id, nc_uuid, nc_medical_institution_code, nc_medical_record_no," \
                            "nc_discharge_time, nc_admission_time, nc_table_name, nc_column_name, nc_table_uuid," \
                            "nc_page, nc_page_uuid, nc_start_index, nc_end_index, nc_content, nc_md5_sum, nc_global_id"\
                            " from nc_document where nc_uuid in ({})".format(query_sql)
    get_document_list_sql = generate_dataset_screen_sql(dataset_id, get_document_list_sql)
    cursor.execute(get_document_list_sql)
    document_list = []
    for ele in cursor.fetchall():
        document = Document(ele[0], ele[1], ele[2], ele[3], ele[4], ele[5], ele[6], ele[7], ele[8], ele[9], ele[10],
                            ele[11], ele[12], ele[13], ele[14])
        document.global_id = ele[14]
        document_list.append(document)

    return document_list


def give_global_id(cursor, document_list, source_table_name='nc_mpi_relation', dataset_id=None):
    sql = 'select nc_medical_institution_code, nc_medical_record_no, nc_discharge_time, nc_global_id from ' \
                   + source_table_name
    sql = generate_dataset_screen_sql(dataset_id, sql)
    cursor.execute(sql)
    relation_key_dict = {}
    for ele in cursor:
        relation_key = ele[0] + "|" + ele[1] + "|"
        if type(ele[2]) == str:
            relation_key += ele[2].replace("-", "").replace(" ", "").replace(":", "")
        else:
            relation_key += ele[2].strftime('%Y%m%d%H%M%S')
        relation_key_dict[relation_key] = ele[3]

    for document in document_list:
        relation_key = document.package_relation_key()
        if relation_key in relation_key_dict:
            document.global_id = relation_key_dict[relation_key]


def calculate_document_list(document_list):
    # 创建md5对象
    md5_hash = hashlib.md5()
    for document in document_list:
        # 使用正则表达式替换掉所有非中文、非英文、非数字字符
        cleaned_string = re.sub(r'[^\u4e00-\u9fffA-Za-z0-9]', '', document.content)
        if cleaned_string:
            md5_hash.update(cleaned_string.encode('utf-8'))
            document.md5_sum = md5_hash.hexdigest()
        key = document.medical_institution_code + document.medical_record_no + document.discharge_time + document.table_name + document.column_name + document.md5_sum
        md5_hash.update(key.encode('utf-8'))
        document.uuid = md5_hash.hexdigest()


def extract_all_relation_key(cursor, dataset_id=None):
    get_all_relation_key_sql = "select nc_medical_institution_code, nc_medical_record_no, nc_discharge_time, " \
                               "nc_admission_time from nc_medical_record_first_page where nc_hedge = 0 and " \
                               "nc_data_status != 99 and nc_data_report_type = 1"
    get_all_relation_key_sql = generate_dataset_screen_sql(dataset_id, get_all_relation_key_sql)
    get_all_relation_key_sql += " order by nc_rid"
    cursor.execute(get_all_relation_key_sql)
    relation_key_list = []
    for ele in cursor.fetchall():
        relation_key_list.append(
            RelationKey(ele[0], ele[1], ele[2].strftime('%Y-%m-%d %H:%M:%S'), ele[3].strftime('%Y-%m-%d %H:%M:%S')))

    return relation_key_list


def extract_relation_key_list_by_global_id(cursor, global_id, dataset_id=None):
    get_relation_key_list_by_global_id_sql = "select nc_medical_institution_code, nc_medical_record_no, " \
                                             "nc_discharge_time, nc_admission_time from nc_mpi_relation where " \
                                             "nc_global_id = '{}'".format(global_id)
    get_relation_key_list_by_global_id_sql = generate_dataset_screen_sql(dataset_id,
                                                                         get_relation_key_list_by_global_id_sql)
    get_relation_key_list_by_global_id_sql += " order by nc_admission_time"
    cursor.execute(get_relation_key_list_by_global_id_sql)
    relation_key_list = []
    for ele in cursor.fetchall():
        relation_key_list.append(
            RelationKey(ele[0], ele[1], ele[2].strftime('%Y-%m-%d %H:%M:%S'), ele[3].strftime('%Y-%m-%d %H:%M:%S')))
    return relation_key_list


def generate_order_sql(sql):
    for table_name, order_info in ORDER_TABLE_DICT.items():
        if sql.count(table_name):
            return order_info['tableUuid'], order_info['pageUuid'], sql + order_info['orderSql']
    return '', '', sql


def sentence_count(sentence_list):
    count_dict = Counter(sentence_list)
    for sentence, count in sorted(count_dict.items(), key=lambda t: t[1], reverse=True):
        print(sentence, count)


def random_pick_relation_key(relation_key_dict, num_to_pick):
    selected_relation_key_list = []
    avg_to_pick = num_to_pick // len(relation_key_dict.keys())
    left_relation_key_list = []
    for _, relation_key_list in relation_key_dict.items():
        if len(relation_key_list) <= avg_to_pick:
            selected_relation_key_list.extend(relation_key_list)
        else:
            random_list = random.sample(relation_key_list, avg_to_pick)
            random_set = set(random_list)
            selected_relation_key_list.extend(random_list)
            left_relation_key_list.extend([relation_key for relation_key in relation_key_list
                                           if relation_key not in random_set])
    selected_relation_key_list.extend(
        random.sample(left_relation_key_list, num_to_pick - len(selected_relation_key_list)))
    return selected_relation_key_list



import re
import time
from datetime import datetime
from dateutil.relativedelta import relativedelta

num_dict = {'一': '1', '两': '2', '二': '2', '三': '3', '四': '4', '五': '5', '六': '6', '七': '7', '八': '8', '九': '9',
            '十': '10', '壹': '1', '贰': '2', '叁': '3', '肆': '4', '伍': '5', '陆': '6', '柒': '7', '捌': '8', '玖': '9',
            '拾': '10',
            '０': '0', '１': '1', '２': '2', '３': '3', '４': '4', '５': '5', '６': '6', '７': '7', '８': '8', '９': '9',
            '0': '0', '1': '1', '2': '2', '3': '3', '4': '4', '5': '5', '6': '6', '7': '7', '8': '8', '9': '9'
            }
keyword_dict = {'年': 0, '月': 1, '周': 2, '天': 3, '日': 3}


def current_date():
    return time.strftime('%Y-%m-%d', time.localtime())


def current_time():
    return time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())


def convert_compact_timestamp_to_standard(timestamp):
    if len(timestamp) != 14:
        return None
    return timestamp[:4] + '-' + timestamp[4:6] + '-' + timestamp[6:8] + ' ' + timestamp[8:10] + ':' + timestamp[10:12] \
           + ':' + timestamp[12:14]


def convert_standard_timestamp_to_compact(timestamp):
    compact_timestamp = ""
    for ch in timestamp:
        if ch.isdigit():
            compact_timestamp += ch
    return compact_timestamp


def _parse_time_seg_with_dot(time_seg):
    add_dict = {'年余': 93, '月余': 14, '周余': 3}
    unit_dict = {'年': 365, '月': 31, '周': 7, '天': 1, '日': 1}
    days = None
    for keyword, unit in unit_dict.items():
        if time_seg.count(keyword) > 0:
            days = unit
            break
    if not days:
        return None

    num = ""
    for ch in time_seg:
        if ch.isdigit() or ch == '.':
            num += ch
        else:
            break
    if not num:
        return None

    days = int(float(num) * days)
    for keyword, add_days in add_dict.items():
        if time_seg.count(keyword) > 0:
            days += add_days
            break

    return [0, 0, 0, days]


def _parse_time_seg_with_hour(time_seg):
    num = ""
    for ch in time_seg:
        if ch.isdigit():
            num += ch
        else:
            break
    if not num:
        return None

    return [0, 0, 0, int(num) // 24]


def _parse_time_seg(time_seg):
    # 年、月、周、天
    result = [0, 0, 0, 0]

    # 上一个解析成功的关键字
    last = None
    # 容忍度
    tolerate = 1
    cur = ""
    for i, ch in enumerate(time_seg):
        if ch == '+' or ch == '-':
            if last is None:
                # +出现在前面
                if i + 1 < len(time_seg) and time_seg[i + 1] in keyword_dict:
                    last = keyword_dict[time_seg[i + 1]]
            if last is not None:
                if ch == '+':
                    # x年+
                    if last == 0:
                        result[1] += 3
                    # x月+
                    if last == 1:
                        result[2] += 2
                    # x周+
                    if last == 2:
                        result[3] += 3
                if ch == '-':
                    # x年-,减1年，加9个月
                    if last == 0 and result[0] > 0:
                        result[0] -= 1
                        result[1] += 9
                    # x月-，减一个月，加两周
                    if last == 1 and result[1] > 0:
                        result[1] -= 1
                        result[2] += 2
                    # x周-，减一周，加三天
                    if last == 2 and result[2] > 0:
                        result[2] -= 1
                        result[3] += 3

        elif ch == '半':
            if last is None:
                # 半字出现在前面
                if i + 1 < len(time_seg) and time_seg[i + 1] in keyword_dict:
                    last = keyword_dict[time_seg[i + 1]]
            if last is not None:
                # 半年
                if last == 0:
                    result[1] += 6
                # 半月
                if last == 1:
                    result[2] += 2
                # 半周
                if last == 2:
                    result[3] += 3
                last += 1
        elif ch == '余':
            # 余字出现在前面
            if i + 1 < len(time_seg) and time_seg[i + 1] in keyword_dict:
                last = keyword_dict[time_seg[i + 1]]
            if last is not None:
                # 年余
                if last == 0:
                    result[1] += 3
                # 月余
                if last == 1:
                    result[2] += 2
                # 周余
                if last == 2:
                    result[3] += 3

        elif ch in num_dict:
            if ch == '十' or ch == '拾':
                if cur != '':
                    if i + 1 < len(time_seg) and time_seg[i + 1] not in num_dict:
                        cur += '0'
                elif i + 1 < len(time_seg):
                    if time_seg[i + 1] in num_dict:
                        cur += '1'
                    else:
                        cur += num_dict[ch]
            else:
                cur += num_dict[ch]
        elif ch in keyword_dict and cur:
            result[keyword_dict[ch]] = int(cur)
            last = keyword_dict[ch]
            cur = ''
            tolerate = 1
        elif ch == '多' or ch == '个':
            continue
        elif tolerate:
            tolerate -= 1
        else:
            break
    return result


def _clean_time_seg(time_seg, current_year):
    if re.match("\d{1,2}月\d{1,2}日", time_seg):
        return None
    if time_seg.find("+") != -1 and time_seg.find("余") != -1:
        time_seg = time_seg.replace("余", "")
    if time_seg.count('同年'):
        time_seg = time_seg.replace('同', str(current_year))
    if time_seg.count('今年'):
        time_seg = time_seg.replace('今', str(current_year))
    if time_seg.count('去年'):
        time_seg = time_seg.replace('去', str(current_year - 1))
    if time_seg.count('前年'):
        time_seg = time_seg.replace('前', str(current_year - 2))
    return time_seg


def _timeseg_rationality_check(time_period_list):
    if time_period_list[0] > 25 or time_period_list[1] > 100 or time_period_list[2] > 100:
        return False
    return True


def _date_substract(start_date, time_period_list):
    start_date -= relativedelta(years=time_period_list[0])
    start_date -= relativedelta(months=time_period_list[1])
    start_date -= relativedelta(weeks=time_period_list[2])
    start_date -= relativedelta(days=time_period_list[3])
    return start_date


def _date_addition(start_date, time_period_list):
    start_date += relativedelta(years=time_period_list[0])
    start_date += relativedelta(months=time_period_list[1])
    start_date += relativedelta(weeks=time_period_list[2])
    start_date += relativedelta(days=time_period_list[3])
    return start_date


def _clean_timestamp(timestamp):
    timestamp = timestamp.strip()
    if timestamp.isdigit() and len(timestamp) == 4:
        return timestamp[:2] + '-' + timestamp[2:]
    if timestamp.isdigit() and len(timestamp) == 8:
        return timestamp[:4] + '-' + timestamp[4:6] + '-' + timestamp[6:]
    if timestamp.count('年初'):
        return timestamp[:4] + "-" + '01'
    if timestamp.count('年中'):
        return timestamp[:4] + "-" + '06'
    if timestamp.count('年底'):
        return timestamp[:4] + "-" + '12'
    if timestamp.count('年尾'):
        return timestamp[:4] + "-" + '12'
    if timestamp.count('年末'):
        return timestamp[:4] + "-" + '12'
    if timestamp.count('年终'):
        return timestamp[:4] + "-" + '12'
    return timestamp.replace("年", "-").replace("月", "-").replace(".", "-").replace("/", "-").replace('日', '').replace(' ', '-').replace('至', '-')


def _year_rationality_check(year):
    return len(year) == 4 and year.isdigit() and 1997 <= int(year) <= 2024


def _month_rationality_check(month):
    return month.isdigit() and 1 <= int(month) <= 12


def _day_rationality_check(day):
    return day.isdigit() and 1 <= int(day) <= 31


def _get_month(ymd_list):
    if len(ymd_list) >= 2:
        month = ymd_list[1]
        if len(ymd_list[1]) == 1:
            month = '0' + month
        return month
    return None


def _get_day(ymd_list):
    if len(ymd_list) >= 3:
        day = ymd_list[2]
        if len(ymd_list[2]) == 1:
            day = '0' + day
        return day
    return None


def parse_time_period(time_seg):
    if time_seg in ['今天', '今日', '当天', '当日', '今晚', '今早']:
        return [0, 0, 0, 0]
    if time_seg in ['昨天', '昨日', '昨晚', '昨早', '近日']:
        return [0, 0, 0, 1]
    if time_seg in ['前天', '前日', '前晚', '前早']:
        return [0, 0, 0, 2]
    elif time_seg.find("时") != -1:
        time_period_list = _parse_time_seg_with_hour(time_seg)
    elif time_seg.find(".") != -1:
        time_period_list = _parse_time_seg_with_dot(time_seg)
    else:
        time_period_list = _parse_time_seg(time_seg)
    return time_period_list


def timeseg_parse(time_seg, timestamp, op=_date_substract):
    """
    时间段解析
    :param time_seg:
    """
    current_year = int(timestamp[:4])
    time_seg = _clean_time_seg(time_seg, current_year)
    if not time_seg:
        return None
    time_period_list = parse_time_period(time_seg)
    if not time_period_list:
        return None

    if _year_rationality_check(str(time_period_list[0])):
        # 该时间段为时间戳
        return timestamp_parse('-'.join([str(t) for t in time_period_list if t]))

    if not _timeseg_rationality_check(time_period_list):
        return None
    start_date = datetime.strptime(timestamp, '%Y-%m-%d %H:%M:%S')
    start_date = op(start_date, time_period_list)

    return start_date.strftime('%Y-%m-%d')


def get_month(timestamp, year=None):
    ymd_list = get_ymd_list(timestamp, year)
    if ymd_list and len(ymd_list) > 1:
        return ymd_list[1]
    return None


def get_ymd_list(timestamp, year=None):
    if not timestamp:
        return None
    timestamp = _clean_timestamp(timestamp)
    ymd_list = timestamp.split("-")
    if len(ymd_list) == 3 and len(ymd_list[0]) == 2 and len(ymd_list[1]) == 2 and len(ymd_list[2]) == 2:
        ymd_list[0] = '20' + ymd_list[0]
    ymd_list = [item for item in ymd_list if item]
    if not ymd_list:
        return None
    if not _year_rationality_check(ymd_list[0]):
        if year:
            ymd_list = [year] + ymd_list
        else:
            return None
    month = _get_month(ymd_list)
    if month:
        if _month_rationality_check(month):
            ymd_list[1] = month
        else:
            return None
    day = _get_day(ymd_list)
    if day:
        if _day_rationality_check(day):
            ymd_list[2] = day
        else:
            return None
    return ymd_list


def timestamp_parse(timestamp, year=None):
    ymd_list = get_ymd_list(timestamp, year)
    return '-'.join(ymd_list) if ymd_list else None


def diff_between_dates(t1, t2):
    dt1 = datetime.strptime(t1, "%Y-%m-%d %H:%M:%S")
    dt2 = datetime.strptime(t2, "%Y-%m-%d %H:%M:%S")

    delta = relativedelta(dt2, dt1)

    years = delta.years
    months = delta.months
    days = delta.days

    return str(years) + "-" + str(months) + "-" + str(days)